@@ -1,38 +1,41 @@
-/* 
- * The contents of this file are subject to the Mozilla Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
- * 
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
- * 
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
+ *
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
+ *
  * The Original Code is the Netscape security libraries.
- * 
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are 
- * Copyright (C) 1994-2000 Netscape Communications Corporation.  All
- * Rights Reserved.
- * 
+ *
+ * The Initial Developer of the Original Code is
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1994-2000
+ * the Initial Developer. All Rights Reserved.
+ *
  * Contributor(s):
- * 
- * Alternatively, the contents of this file may be used under the
- * terms of the GNU General Public License Version 2 or later (the
- * "GPL"), in which case the provisions of the GPL are applicable 
- * instead of those above.  If you wish to allow use of your 
- * version of this file only under the terms of the GPL and not to
- * allow others to use your version of this file under the MPL,
- * indicate your decision by deleting the provisions above and
- * replace them with the notice and other provisions required by
- * the GPL.  If you do not delete the provisions above, a recipient
- * may use your version of this file under either the MPL or the
- * GPL.
- */
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: object.c,v $ $Revision: 1.9 $ $Date: 2002/03/29 18:53:15 $ $Name: FIREFOX_1_0_7_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: object.c,v $ $Revision: 1.15 $ $Date: 2007/12/12 00:41:37 $";
 #endif /* DEBUG */
 
 /*
@@ -166,6 +169,10 @@ nssCKFWObject_Create
   }
 #endif /* NSSDEBUG */
 
+  if( (NSSCKFWToken *)NULL == fwToken ) {
+    *pError = CKR_ARGUMENTS_BAD;
+    return (NSSCKFWObject *)NULL;
+  }
   mdObjectHash = nssCKFWToken_GetMDObjectHash(fwToken);
   if( (nssCKFWHash *)NULL == mdObjectHash ) {
     *pError = CKR_GENERAL_ERROR;
@@ -192,11 +199,7 @@ nssCKFWObject_Create
   }
 
   fwObject->fwToken = fwToken;
-
-  if( (NSSCKFWToken *)NULL != fwToken ) {
-    fwObject->mdToken = nssCKFWToken_GetMDToken(fwToken);
-  }
-
+  fwObject->mdToken = nssCKFWToken_GetMDToken(fwToken);
   fwObject->fwInstance = fwInstance;
   fwObject->mdInstance = nssCKFWInstance_GetMDInstance(fwInstance);
   fwObject->mutex = nssCKFWInstance_CreateMutex(fwInstance, arena, pError);
@@ -233,7 +236,8 @@ nssCKFWObject_Create
 NSS_IMPLEMENT void
 nssCKFWObject_Finalize
 (
-  NSSCKFWObject *fwObject
+  NSSCKFWObject *fwObject,
+  PRBool removeFromHash
 )
 {
   nssCKFWHash *mdObjectHash;
@@ -252,12 +256,16 @@ nssCKFWObject_Finalize
       fwObject->fwToken, fwObject->mdInstance, fwObject->fwInstance);
   }
 
-  mdObjectHash = nssCKFWToken_GetMDObjectHash(fwObject->fwToken);
-  if( (nssCKFWHash *)NULL != mdObjectHash ) {
-    nssCKFWHash_Remove(mdObjectHash, fwObject->mdObject);
-  }
+  if (removeFromHash) {
+    mdObjectHash = nssCKFWToken_GetMDObjectHash(fwObject->fwToken);
+    if( (nssCKFWHash *)NULL != mdObjectHash ) {
+      nssCKFWHash_Remove(mdObjectHash, fwObject->mdObject);
+    }
+ }
 
-  nssCKFWSession_DeregisterSessionObject(fwObject->fwSession, fwObject);
+  if (fwObject->fwSession) {
+    nssCKFWSession_DeregisterSessionObject(fwObject->fwSession, fwObject);
+  }
   nss_ZFreeIf(fwObject);
 
 #ifdef DEBUG
@@ -298,7 +306,9 @@ nssCKFWObject_Destroy
     nssCKFWHash_Remove(mdObjectHash, fwObject->mdObject);
   }
 
-  nssCKFWSession_DeregisterSessionObject(fwObject->fwSession, fwObject);
+  if (fwObject->fwSession) {
+    nssCKFWSession_DeregisterSessionObject(fwObject->fwSession, fwObject);
+  }
   nss_ZFreeIf(fwObject);
 
 #ifdef DEBUG
@@ -689,6 +699,7 @@ NSS_IMPLEMENT CK_RV
 nssCKFWObject_SetAttribute
 (
   NSSCKFWObject *fwObject,
+  NSSCKFWSession *fwSession,
   CK_ATTRIBUTE_TYPE attribute,
   NSSItem *value
 )
@@ -716,7 +727,7 @@ nssCKFWObject_SetAttribute
     a.pValue = value->data;
     a.ulValueLen = value->size;
 
-    newFwObject = nssCKFWSession_CopyObject(fwObject->fwSession, fwObject,
+    newFwObject = nssCKFWSession_CopyObject(fwSession, fwObject,
                     &a, 1, &error);
     if( (NSSCKFWObject *)NULL == newFwObject ) {
       if( CKR_OK == error ) {
@@ -767,13 +778,15 @@ nssCKFWObject_SetAttribute
        * New one is a session object, except since we "stole" the fwObject, it's
        * not in the list.  Add it.
        */
-      nssCKFWSession_RegisterSessionObject(fwObject->fwSession, fwObject);
+      nssCKFWSession_RegisterSessionObject(fwSession, fwObject);
     } else {
       /*
        * New one is a token object, except since we "stole" the fwObject, it's
        * in the list.  Remove it.
        */
-      nssCKFWSession_DeregisterSessionObject(fwObject->fwSession, fwObject);
+      if (fwObject->fwSession) {
+        nssCKFWSession_DeregisterSessionObject(fwObject->fwSession, fwObject);
+      }
     }
 
     /*
